package net.eduvax.sched;

import net.eduvax.util.SortedList;

/**
 * Simple sequencer.
 * Use simple 
 */
public class Sequencer implements Runnable {
	private boolean _run;
	private SortedList<Schedulable> _tQueue;
	private Clock _clock;
	
	public Sequencer() {
		_tQueue=new SortedList<Schedulable>();
		_clock=new Clock();
	}

	public void run() {
		_run=true;
		try {
			while (_run) {
				Schedulable toRun=null;
				synchronized(this) {
					if (!_tQueue.isEmpty()) {
						toRun=_tQueue.head();
						_clock.setNextTick(toRun.getTrigTime());
					}
					else {
						_clock.setNextTick(-1);
					}
				}
				_clock.waitTick();
				synchronized(this) {
					toRun=_tQueue.popHead();
				}
				if (toRun!=null) {
					long trigTime=toRun.getTrigTime();
					toRun.run();
					if (toRun.getTrigTime()!=trigTime) {
						// TTR has changed its trig time to a later date,
						// then it must be scheduled once again.
						schedule(toRun);
					}
				}
			}
		}
		catch (InterruptedException ex) {
			ex.printStackTrace();
		}
	}

	public void stop() {
		synchronized(this) {
			_run=false;
			_clock.cancelWait();
		}
	}

	public void schedule(Schedulable toSchedule) {
		synchronized(this) {
			_tQueue.add(toSchedule);
			if (_tQueue.head()==toSchedule) {
				_clock.setNextTick(toSchedule.getTrigTime());
			}
		}
	}

	public Schedulable schedule(long time,long period,Runnable toSchedule) {
		Schedulable s=new Schedulable(time,period,toSchedule);
		schedule(s);
		return s;
	}

	public Schedulable schedule(long time,Runnable toSchedule) {
		return schedule(time,0,toSchedule);
	}

	public void unschedule(Schedulable toUnschedule) {
		synchronized(this) {
			boolean toUnschedIsFirst=_tQueue.head()==toUnschedule;
			_tQueue.remove(toUnschedule);
			if (toUnschedIsFirst) {
				_clock.cancelWait();
			}
		}
	}
}
