/*  $Id: HandlerList.java 1035 2010-06-15 13:08:33Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2010 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.Iterator;
import java.util.Vector;
/**
 * Handler list.
 * Handle methode invocation is passed to all members of the list.
 * Inner list use is synchronized, so no need to worry about concurrent
 * call of add/remove handler and handle methods.
 * @version $Id: HandlerList.java 1035 2010-06-15 13:08:33Z sdevaux $
 * @author Sebastien Devaux
 */
public class HandlerList<T> implements Handler<T> {
	/**
	 * Create new handler list
	 */
	public HandlerList() {
		handlerList=new Vector<Handler<T>>();
	}
	/**
	 * Add an handler to the list
	 * @param handler handler to add
	 */
	public void add(Handler<T> handler){
		synchronized(handlerList) {
			handlerList.add(handler);
		}
	}
	/**
	 * Remove one handler from the list
	 * @param handler handler to remove
	 */
	public void remove(Handler<T> handler) {
		synchronized(handlerList) {
			handlerList.remove(handler);
		}
	}
	// Handle interface implementation
	public void handle(T obj) {
		synchronized(handlerList) {
			Iterator<Handler<T>> it=handlerList.iterator();
			while (it.hasNext()) {
				it.next().handle(obj);
			}
		}
	}
	/** handler collection
	 * @djin.relation aggregation,-,*,Handler
	 */
	private Vector<Handler<T>> handlerList; 
}
