/*  $Id: SaxParser.java 1040 2010-06-15 13:36:52Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2002 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.xml;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.Reader;
import org.xml.sax.ContentHandler;
import org.xml.sax.ErrorHandler;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.helpers.XMLReaderFactory;

/**
 * Sax parser helper, manage sax parsing just by giving filepath
 * and sax handler.
 * @version $Id: SaxParser.java 1040 2010-06-15 13:36:52Z sdevaux $
 * @author Sebastien Devaux	
 */
public class SaxParser {
	/**
	 *
	 */
	protected SaxParser() throws SAXException {
		_xmlReader=XMLReaderFactory.createXMLReader();
	}
	/**
	 * create new parser for xml file.
	 * @param filepath file to parse path
	 * @param handler sax events handler
	 */
	public static SaxParser getFileParser(String filepath,
										  DefaultHandler handler)
								throws FileNotFoundException, SAXException {
		return getFileParser(filepath,handler,handler);
	}
	/**
	 * create new parser for xml file.
	 * @param filepath file to parse path
	 * @param content sax content events handler
	 * @param error sax error events handler
	 */
	public static SaxParser getFileParser(String filepath,
										  ContentHandler content,
										  ErrorHandler error) 
								throws FileNotFoundException, SAXException {
		SaxParser sx=new SaxParser();
		sx._reader=new FileReader(filepath);
		sx._xmlReader.setContentHandler(content);
		sx._xmlReader.setErrorHandler(error);
		return sx;
	}										  
	/**
	 * create new parser for xml stream.
	 * @param input stream to parse
	 * @param handler sax events handler
	 */
	public static SaxParser getInputStreamParser(InputStream input,
										  DefaultHandler handler)
								throws FileNotFoundException, SAXException {
		return getInputStreamParser(input,handler,handler);
	}
	/**
	 * create new parser for xml stream.
	 * @param filepath stream to parse
	 * @param content sax content events handler
	 * @param error sax error events handler
	 */
	public static SaxParser getInputStreamParser(InputStream input,
										  ContentHandler content,
										  ErrorHandler error) 
								throws SAXException {
		SaxParser sx=new SaxParser();
		sx._reader=new InputStreamReader(input);
		sx._xmlReader.setContentHandler(content);
		sx._xmlReader.setErrorHandler(error);
		return sx;
	}										  
	/**
	 * parse xml stream.
	 */
	public void parse() throws IOException, SAXException {
		_xmlReader.parse(new InputSource(_reader));
	}
	/** insternal XML reader. */
	private XMLReader _xmlReader;
	/** source xml stream reader. */
	private Reader _reader;
}


